
///////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify  //
// it under the terms of the version 3 of the GNU General Public License //
// as published by the Free Software Foundation.                         //
//                                                                       //
// This program is distributed in the hope that it will be useful, but   //
// WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
// General Public License for more details.                              //
//                                                                       //
// You should have received a copy of the GNU General Public License     //
// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
//                                                                       //
// Written by Francois Fleuret, (C) IDIAP                                //
// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
///////////////////////////////////////////////////////////////////////////

#include "pose_cell.h"
#include "global.h"
#include "tools.h"

bool PoseCell::contains(Pose *pose) {
  return
    _head_xc.contains(pose->_head_xc) &&
    _head_yc.contains(pose->_head_yc) &&
    _head_radius.contains(pose->_head_radius) &&
    _body_xc.contains(pose->_body_xc) &&
    _body_yc.contains(pose->_body_yc);
}

bool PoseCell::negative_for_train(Pose *pose) {
  const scalar_t r_tolerance = 3;
  scalar_t r = pose->_head_radius;
  return
    r <= _head_radius.min / r_tolerance ||
    r >= _head_radius.max * r_tolerance ||
    pose->_head_xc <= _head_xc.min - r ||
    pose->_head_xc >= _head_xc.max + r ||
    pose->_head_yc <= _head_yc.min - r ||
    pose->_head_yc >= _head_yc.max + r;
}

void PoseCell::get_centroid(Pose *pose) {
  pose->_bounding_box_xmin = -1;
  pose->_bounding_box_ymin = -1;
  pose->_bounding_box_xmax = -1;
  pose->_bounding_box_ymax = -1;
  pose->_head_radius = sqrt(_head_radius.min * _head_radius.max);
  pose->_head_xc = _head_xc.middle();
  pose->_head_yc = _head_yc.middle();
  pose->_body_xc = _body_xc.middle();
  pose->_body_yc = _body_yc.middle();
}

void PoseCell::print(ostream *out) {
  (*out) << "  _head_xc " << _head_xc << endl;
  (*out) << "  _head_yc " << _head_yc << endl;
  (*out) << "  _head_radius " << _head_radius << endl;
  (*out) << "  _head_tilt " << _head_tilt << endl;
  (*out) << "  _body_xc " << _body_xc << endl;
  (*out) << "  _body_yc " << _body_yc << endl;
}
