
///////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify  //
// it under the terms of the version 3 of the GNU General Public License //
// as published by the Free Software Foundation.                         //
//                                                                       //
// This program is distributed in the hope that it will be useful, but   //
// WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
// General Public License for more details.                              //
//                                                                       //
// You should have received a copy of the GNU General Public License     //
// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
//                                                                       //
// Written by Francois Fleuret, (C) IDIAP                                //
// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
///////////////////////////////////////////////////////////////////////////

#ifndef POSE_CELL_SET_H
#define POSE_CELL_SET_H

#include "pose_cell.h"
#include "labelled_image.h"

class PoseCellSet {
protected:
  // This is a bit ugly. Notice that all the code is written in such a
  // way that we never have more than one such PoseCellSet in memory
  // at any moment
  static const int max_nb_cells = 500000;

  int _max_nb;
  int _nb_added;
  PoseCell *_cells;

public:

  PoseCellSet();
  ~PoseCellSet();

  inline int nb_cells() { return _nb_added; }

  inline PoseCell *get_cell(int k) {
    ASSERT(k >= 0 && k < _nb_added);
    return _cells + k;
  }

  PoseCell *get_containing_cell(Pose *p);

  void erase_content();
  void add_cell(PoseCell *cell);
};

#endif
